// Tools for manipulating the DOM

'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global translate */


// DOM contains static methods to assist in creating HTML elements
// that is specific to AdBlock UIs. Relies on jQuery to be in the
// global namespace as `$`.
class DOM {
  // a generates an <a></a> link with the given content, target, and optional classes.
  //
  // If the content provided is:
  //
  // * `string` - the link text is set to the i18n key translation.
  // * otherwise - assumes the content is jQuery elements to be appended to the link
  //
  // If a target is provided it can one of the following:
  //
  // * `string` - the href for the link is set to the target
  // * `undefined` or `null` - the href for the link is set to '#'
  // * `function` - the href is set to `#` and the function is set as the click handler
  static a(content, target, classes) {
    let $link = $('<a></a>');
    switch (typeof content) {
      case 'string':
        $link = DOM.fill($link, content, classes);
        break;
      default:
        if (classes) {
          $link.addClass(classes);
        }
        $link.append(content);
    }
    switch (typeof target) {
      case 'string':
        $link.attr('href', target);
        break;
      case 'function':
        $link.click((e) => {
          e.stopPropagation();
          e.preventDefault();
          target(e);
        });
      // fallthrough
      default:
        $link.attr('href', '#');
    }
    return $link;
  }

  // p generates a <p></p> paragraph element containing the given i18n and
  // assigned an optional class. Passing an object provides more options
  // including avoiding i18n by passing in the text.
  static p(key, classes) {
    return DOM.fill('<p></p>', key, classes);
  }

  // h1 generates a <h1></h1> element filled with default attributes and text
  // (see DOM.fill() for more).
  static h1(key, classes) {
    return DOM.fill('<h1></h1>', key, classes);
  }

  // fill in an element with the given i18n and assigned optional class.
  // Passing an object instead of key provides more options:
  //
  // * i18n - the i18n message key to assign and append text.
  // * text - the text to append to the element
  // * classes - one or more classes (space separated) to assign to the element.
  static fill($element, key, classes) {
    let $el;
    if (typeof $element === 'string') {
      $el = $($element);
    } else {
      $el = $element;
    }
    switch (typeof key) {
      case 'string':
        $el.attr('i18n', key);
        if (classes) {
          $el.addClass(classes);
        }
        break;
      case 'object':
        if (key.i18n) {
          $el.attr('i18n', key.i18n);
          $el.text(translate(key.i18n));
        }
        if (key.text) {
          $el.text(key.text);
        }
        if (key.classes) {
          $el.addClass(key.classes);
        }
        break;
      default:
        // This should never happen but if it does we just don't do anything with the $element
    }
    return $el;
  }

  // icon generates an icon <i></i> element with the given name. If the
  // name is an array, the icons are stacked using md-stack. Use an optional
  // stackClasses array to match the name array for assigning different classes
  // to each icon in the stack. By default, the first icon is assigned `md-stack-1x`
  // and any others are `md-stack-2x`.
  static icon(name, classes, stackClasses) {
    if (!Array.isArray(name)) {
      const $icon = $('<i></i>');
      $icon.addClass('material-icons');
      if (classes) {
        $icon.addClass(classes);
      }
      $icon.text(name);
      return $icon;
    }
    // Otherwise, this is a stack
    const $icons = [];
    const cleanStackClasses = [];
    if (!Array.isArray(stackClasses) || stackClasses.length !== name.length) {
      for (let i = 0; i < name.length; i++) {
        if (i === 0) {
          cleanStackClasses.push('md-stack-1x');
        } else {
          cleanStackClasses.push('md-stack-2x');
        }
      }
    } else {
      cleanStackClasses.push(...stackClasses);
    }
    for (let i = 0; i < name.length; i++) {
      const $icon = $('<i></i>');
      $icon.addClass(['material-icons', cleanStackClasses[i]]);
      $icon.text(name[i]);
      $icons.push($icon);
    }
    const $stack = $('<span></span>');
    $stack.addClass('md-stack');
    if (classes) {
      $stack.addClass(classes);
    }
    $stack.append($icons);
    return $stack;
  }

  // img generates an <img/> element with the classes and i18n alt text.
  static img(src, classes, alt) {
    const $img = $('<img/>');
    if (src) {
      $img.attr('src', src);
    }
    if (classes) {
      $img.addClass(classes);
    }
    if (alt) {
      $img.attr('i18n', alt);
      $img.attr('alt', translate(alt));
    }
    return $img;
  }

  // div creates a <div></div> element with a optional classes assigned and
  // any number of children. Additional options can be passed in a third parameter:
  //
  // * `id` - sets the id attribute of the div
  // * `click` - sets a click handler for the div
  // * `ready` - handler that is called with this div when it's finished building.
  static div(classes, children, options) {
    const $div = $('<div></div>');
    if (classes) {
      $div.addClass(classes);
    }
    if (children) {
      $div.append(children);
    }
    if (options) {
      if (options.id) {
        $div.attr('id', options.id);
      }
      if (options.click) {
        $div.click(options.click);
      }
      if (options.ready) {
        options.ready($div);
      }
    }
    return $div;
  }

  // addClassIf the flag is true then adds the classes provided.
  // Short cut for:
  //
  // ```js
  // if (flag) {
  //   $el.addClass(classes);
  // }
  // ```
  static addClassIf($el, flag, classes) {
    if (flag) {
      $el.addClass(classes);
    }
  }
}
